@@ -50,6 +50,7 @@ extern struct ec_backend_common backend_isa_l_rs_vand;
 extern struct ec_backend_common backend_shss;
 extern struct ec_backend_common backend_liberasurecode_rs_vand;
 extern struct ec_backend_common backend_isa_l_rs_cauchy;
+extern struct ec_backend_common backend_libphazr;
 
 ec_backend_t ec_backends_supported[] = {
     (ec_backend_t) &backend_null,
@@ -60,6 +61,7 @@ ec_backend_t ec_backends_supported[] = {
     (ec_backend_t) &backend_shss,
     (ec_backend_t) &backend_liberasurecode_rs_vand,
     (ec_backend_t) &backend_isa_l_rs_cauchy,
+    (ec_backend_t) &backend_libphazr,
     NULL,
 };
 
@@ -587,6 +589,13 @@ int liberasurecode_decode(int desc,
         goto out;
     }
 
+    if (fragment_len < sizeof(fragment_header_t)) {
+        log_error("Fragments not long enough to include headers! "
+                  "Need %zu, but got %lu.", sizeof(fragment_header_t),
+                  (unsigned long)fragment_len);
+        ret = -EBADHEADER;
+        goto out;
+    }
     for (i = 0; i < num_fragments; ++i) {
         /* Verify metadata checksum */
         if (is_invalid_fragment_header(
@@ -597,8 +606,8 @@ int liberasurecode_decode(int desc,
         }
     }
 
-    if (instance->common.id != EC_BACKEND_SHSS) {
-        /* shss (ntt_backend) must force to decode */
+    if (instance->common.id != EC_BACKEND_SHSS && instance->common.id != EC_BACKEND_LIBPHAZR) {
+        /* shss (ntt_backend) & libphazr backend must force to decode */
         // TODO: Add a frag and function to handle whether the backend want to decode or not.
         /*
          * Try to re-assebmle the original data before attempting a decode
@@ -1185,8 +1194,8 @@ int liberasurecode_verify_stripe_metadata(int desc,
 /**
  * This computes the aligned size of a buffer passed into
  * the encode function.  The encode function must pad fragments
- * to be algined with the word size (w) and the last fragment also
- * needs to be aligned.  This computes the sum of the algined fragment
+ * to be aligned with the word size (w) and the last fragment also
+ * needs to be aligned.  This computes the sum of the aligned fragment
  * sizes for a given buffer to encode.
  */
 int liberasurecode_get_aligned_data_size(int desc, uint64_t data_len)
@@ -1209,8 +1218,8 @@ int liberasurecode_get_aligned_data_size(int desc, uint64_t data_len)
 
     alignment_multiple = k * word_size;
 
-    ret = (int) ceill( (double)
-            data_len / alignment_multiple) * alignment_multiple;
+    ret = ((data_len + alignment_multiple - 1) / alignment_multiple)
+            * alignment_multiple;
 
 out:
     return ret;
@@ -1232,7 +1241,11 @@ int liberasurecode_get_fragment_size(int desc, int data_len)
     if (NULL == instance)
         return -EBACKENDNOTAVAIL;
     int aligned_data_len = get_aligned_data_size(instance, data_len);
-    int size = (aligned_data_len / instance->args.uargs.k) + instance->common.backend_metadata_size;
+    int blocksize = aligned_data_len / instance->args.uargs.k;
+    int metadata_size = instance->common.ops->get_backend_metadata_size(
+                                                instance->desc.backend_desc,
+                                                blocksize);
+    int size = blocksize + metadata_size;
 
     return size;
 }
