@@ -42,6 +42,7 @@
 #define ISA_L_RS_CAUCHY_BACKEND "isa_l_rs_cauchy"
 #define SHSS_BACKEND "shss"
 #define RS_VAND_BACKEND "liberasurecode_rs_vand"
+#define LIBPHAZR_BACKEND "libphazr"
 
 typedef void (*TEST_FUNC)();
 
@@ -222,6 +223,13 @@ struct ec_args *liberasurecode_rs_vand_test_args[] = {
                &liberasurecode_rs_vand_48_args,
                NULL };
 
+struct ec_args libphazr_args = {
+    .k = 4,
+    .m = 4,
+};
+
+struct ec_args *libphazr_test_args[] = { &libphazr_args, NULL };
+
 struct ec_args **all_backend_tests[] = {
                null_test_args,
                flat_xor_test_args,
@@ -230,6 +238,7 @@ struct ec_args **all_backend_tests[] = {
                isa_l_test_args,
                shss_test_args,
                liberasurecode_rs_vand_test_args,
+               libphazr_test_args,
                NULL};
 
 int num_backends()
@@ -288,6 +297,8 @@ char * get_name_from_backend_id(ec_backend_id_t be) {
             return SHSS_BACKEND;
         case EC_BACKEND_LIBERASURECODE_RS_VAND:
             return RS_VAND_BACKEND;
+        case EC_BACKEND_LIBPHAZR:
+            return LIBPHAZR_BACKEND;
         default:
             return "UNKNOWN";
     }
@@ -325,6 +336,9 @@ struct ec_args *create_ec_args(ec_backend_id_t be, ec_checksum_type_t ct, int ba
         case EC_BACKEND_SHSS:
             backend_args_array = shss_test_args;
             break;
+        case EC_BACKEND_LIBPHAZR:
+            backend_args_array = libphazr_test_args;
+            break;
         default:
             return NULL;
     }
@@ -373,6 +387,9 @@ int *create_skips_array(struct ec_args *args, int skip)
 static int create_fake_frags_no_meta(char ***array, int num_frags,
                                      const char *data, int data_len)
 {
+    // N.B. The difference from creat_frags_arry is to creat new
+    // memory allocation and set a copy of data/parity there. The
+    // allocated memory should be maintained by the caller.
     int _num_frags = 0;
     int i = 0;
     char **ptr = NULL;
@@ -401,6 +418,10 @@ static int create_frags_array(char ***array,
                               struct ec_args *args,
                               int *skips)
 {
+    // N.B. this function sets pointer reference to the ***array
+    // from **data and **parity so DO NOT free each value of
+    // the array independently because the data and parity will
+    // be expected to be cleanup via liberasurecode_encode_cleanup
     int num_frags = 0;
     int i = 0;
     char **ptr = NULL;
@@ -431,6 +452,12 @@ out:
     return num_frags;
 }
 
+static void cleanup_avail_frags(char **avail_frags,
+                                int num_frags)
+{
+   while(num_frags > 0) free(avail_frags[--num_frags]);
+   free(avail_frags);
+}
 static int encode_failure_stub(void *desc, char **data,
                                char **parity, int blocksize)
 {
@@ -582,6 +609,7 @@ static void test_encode_invalid_args()
     assert(rc < 0);
     instance->common.ops->encode = orig_encode_func;
 
+    liberasurecode_instance_destroy(desc);
     free(orig_data);
 }
 
@@ -613,6 +641,7 @@ static void test_encode_cleanup_invalid_args()
 
     rc = liberasurecode_encode_cleanup(desc, encoded_data, encoded_parity);
     assert(rc == 0);
+    liberasurecode_instance_destroy(desc);
     free(orig_data);
 }
 
@@ -629,7 +658,10 @@ static void test_decode_invalid_args()
     int *skips = create_skips_array(&null_args, -1);
     char *decoded_data = NULL;
     uint64_t decoded_data_len = 0;
-    const char *fake_data = " ";
+    // fake_data len should be bigger than fragment_header_t for
+    // the verifications
+    int fake_data_len = 1024;
+    char *fake_data = create_buffer(fake_data_len, 'y');
 
     desc = liberasurecode_instance_create(EC_BACKEND_NULL, &null_args);
     if (-EBACKENDNOTAVAIL == desc) {
@@ -641,30 +673,40 @@ static void test_decode_invalid_args()
     // test with invalid fragments (no metadata headers)
     num_avail_frags = create_fake_frags_no_meta(&avail_frags, (null_args.k +
                                                 null_args.m),
-                                                fake_data, strlen(fake_data));
+                                                fake_data, fake_data_len);
     assert(num_avail_frags > 0);
+    free(fake_data);
 
     rc = liberasurecode_decode(desc, avail_frags, num_avail_frags,
-                               strlen(fake_data), 1,
+                               fake_data_len, 1,
                                &decoded_data, &decoded_data_len);
     // no metadata headers w/ force_metadata_checks results in EBADHEADER
     assert(rc == -EBADHEADER);
 
     rc = liberasurecode_decode(desc, avail_frags, num_avail_frags,
-                               strlen(fake_data), 0,
+                               fake_data_len, 0,
                                &decoded_data, &decoded_data_len);
     // no metadata headers w/o force_metadata_checks also results in EBADHEADER
     assert(rc == -EBADHEADER);
 
+    // encoded_fragment_len is too small
+    rc = liberasurecode_decode(desc, avail_frags, num_avail_frags,
+                               1, 1, &decoded_data, &decoded_data_len);
+    assert(rc == -EBADHEADER);
+
+    cleanup_avail_frags(avail_frags, num_avail_frags);
+
     // test with num_fragments < (k)
     num_avail_frags = create_fake_frags_no_meta(&avail_frags, (null_args.k - 1),
                                                 " ", 1);
     assert(num_avail_frags > 0);
     rc = liberasurecode_decode(desc, avail_frags, num_avail_frags,
-                               strlen(fake_data), 1,
+                               fake_data_len, 1,
                                &decoded_data, &decoded_data_len);
     assert(rc == -EINSUFFFRAGS);
 
+    cleanup_avail_frags(avail_frags, num_avail_frags);
+
     rc = liberasurecode_encode(desc, orig_data, orig_data_size,
             &encoded_data, &encoded_parity, &encoded_fragment_len);
     assert(rc == 0);
@@ -692,11 +734,15 @@ static void test_decode_invalid_args()
                                encoded_fragment_len, 1,
                                &decoded_data, NULL);
     assert(rc < 0);
+
     free(skips);
     liberasurecode_encode_cleanup(desc, encoded_data, encoded_parity);
+    liberasurecode_instance_destroy(desc);
+    // N.B. create_frags_array sets pointer reference of either encoded_data
+    // or encoded_parity and they are cleaned up via
+    // liberasurecode_encode_cleanup
     free(avail_frags);
     free(orig_data);
-
 }
 
 static void test_decode_cleanup_invalid_args()
@@ -717,7 +763,7 @@ static void test_decode_cleanup_invalid_args()
 
     rc = liberasurecode_decode_cleanup(desc, NULL);
     assert(rc == 0);
-
+    liberasurecode_instance_destroy(desc);
     free(orig_data);
 }
 
@@ -753,6 +799,7 @@ static void test_reconstruct_fragment_invalid_args()
     assert(rc < 0);
 
     free(out_frag);
+    free(avail_frags);
 
     // Test for EINSUFFFRAGS
     // we have to call encode to get fragments which have valid header.
@@ -766,9 +813,10 @@ static void test_reconstruct_fragment_invalid_args()
     rc = liberasurecode_reconstruct_fragment(desc, encoded_data, 1, encoded_fragment_len, 1, out_frag);
 
     assert(rc == -EINSUFFFRAGS);
+    free(orig_data);
     free(out_frag);
-    free(avail_frags);
     liberasurecode_encode_cleanup(desc, encoded_data, encoded_parity);
+    liberasurecode_instance_destroy(desc);
 }
 
 static void test_fragments_needed_invalid_args()
@@ -797,6 +845,7 @@ static void test_fragments_needed_invalid_args()
 
     rc = liberasurecode_fragments_needed(desc, &frags_to_recon, &frags_to_exclude, NULL);
     assert(rc < 0);
+    liberasurecode_instance_destroy(desc);
 }
 
 static void test_get_fragment_metadata_invalid_args() {
@@ -847,7 +896,8 @@ static void test_verify_stripe_metadata_invalid_args() {
 
     rc = liberasurecode_verify_stripe_metadata(desc, frags, 0);
     assert(rc == -EINVALIDPARAMS);
-
+    liberasurecode_instance_destroy(desc);
+    free(frags);
 }
 
 static void test_get_fragment_partition()
@@ -867,6 +917,8 @@ static void test_get_fragment_partition()
     desc = liberasurecode_instance_create(EC_BACKEND_NULL, &null_args);
     if (-EBACKENDNOTAVAIL == desc) {
         fprintf (stderr, "Backend library not available!\n");
+        free(orig_data);
+        free(skips);
         return;
     }
     assert(desc > 0);
@@ -876,7 +928,19 @@ static void test_get_fragment_partition()
 
     missing = alloc_and_set_buffer(sizeof(char*) * null_args.k, -1);
 
-    for(i = 0; i < null_args.m; i++) skips[i] = 1;
+    for(i = 0; i < null_args.m; i++) {
+        skips[i] = 1;
+        /* get_fragment_partition is going to NULL all the entries in
+         * encoded_data and encoded_parity before populating them again
+         * from avail_frags. Since we're explicitly *not* including these
+         * data frags in avail_frags, free them now.
+         */
+        if (i < null_args.k) {
+            free(encoded_data[i]);
+        } else {
+            free(encoded_parity[i - null_args.k]);
+        }
+    }
     num_avail_frags = create_frags_array(&avail_frags, encoded_data,
                                          encoded_parity, &null_args, skips);
 
@@ -885,11 +949,19 @@ static void test_get_fragment_partition()
     assert(0 == rc);
 
     for(i = 0; i < null_args.m; i++) assert(missing[i] == i);
-    assert(missing[++i] == -1);
+    // Loop already pushed us one past
+    assert(missing[i] == -1);
 
+    free(avail_frags);
     free(missing);
 
-    for(i = 0; i < null_args.m + 1; i++) skips[i] = 1;
+    skips[i] = 1;
+    if (i < null_args.k) {
+        free(encoded_data[i]);
+    } else {
+        free(encoded_parity[i - null_args.k]);
+    }
+
     num_avail_frags = create_frags_array(&avail_frags, encoded_data,
                                          encoded_parity, &null_args, skips);
 
@@ -905,6 +977,7 @@ static void test_get_fragment_partition()
     free(missing);
     free(skips);
     liberasurecode_encode_cleanup(desc, encoded_data, encoded_parity);
+    liberasurecode_instance_destroy(desc);
     free(avail_frags);
     free(orig_data);
 }
@@ -932,10 +1005,8 @@ static void encode_decode_test_impl(const ec_backend_id_t be_id,
     int num_avail_frags = 0;
     char *orig_data_ptr = NULL;
     int remaining = 0;
-    ec_backend_t be = NULL;
 
     desc = liberasurecode_instance_create(be_id, args);
-    be = liberasurecode_backend_instance_get_by_desc(desc);
 
     if (-EBACKENDNOTAVAIL == desc) {
         fprintf (stderr, "Backend library not available!\n");
@@ -970,14 +1041,14 @@ static void encode_decode_test_impl(const ec_backend_id_t be_id,
 
         fragment_metadata_t metadata = header->meta;
         assert(metadata.idx == i);
-        assert(metadata.size == encoded_fragment_len - frag_header_size - be->common.backend_metadata_size);
+        assert(metadata.size == encoded_fragment_len - frag_header_size - metadata.frag_backend_metadata_size);
         assert(metadata.orig_data_size == orig_data_size);
         assert(metadata.backend_id == be_id);
         assert(metadata.chksum_mismatch == 0);
         data_ptr = frag + frag_header_size;
         cmp_size = remaining >= metadata.size ? metadata.size : remaining;
-        // shss doesn't keep original data on data fragments
-        if (be_id != EC_BACKEND_SHSS) {
+        // shss & libphazr doesn't keep original data on data fragments
+        if (be_id != EC_BACKEND_SHSS && be_id != EC_BACKEND_LIBPHAZR) {
             assert(memcmp(data_ptr, orig_data_ptr, cmp_size) == 0);
         }
         remaining -= cmp_size;
@@ -1000,10 +1071,7 @@ static void encode_decode_test_impl(const ec_backend_id_t be_id,
     rc = liberasurecode_decode_cleanup(desc, decoded_data);
     assert(rc == 0);
 
-    if (desc) {
-        assert(0 == liberasurecode_instance_destroy(desc));
-    }
-
+    assert(0 == liberasurecode_instance_destroy(desc));
     free(orig_data);
     free(avail_frags);
 }
@@ -1075,11 +1143,12 @@ static void reconstruct_test_impl(const ec_backend_id_t be_id,
         rc = liberasurecode_reconstruct_fragment(desc, avail_frags, num_avail_frags, encoded_fragment_len, i, out);
         assert(rc == 0);
         assert(memcmp(out, cmp, encoded_fragment_len) == 0);
+        free(avail_frags);
     }
     free(orig_data);
     free(out);
-    free(avail_frags);
     liberasurecode_encode_cleanup(desc, encoded_data, encoded_parity);
+    liberasurecode_instance_destroy(desc);
 }
 
 static void test_fragments_needed_impl(const ec_backend_id_t be_id,
@@ -1203,6 +1272,7 @@ static void test_fragments_needed_impl(const ec_backend_id_t be_id,
         assert(is_valid_fragment == 1);
         i++;
     }
+    liberasurecode_instance_destroy(desc);
     free(fragments_to_reconstruct);
     free(fragments_to_exclude);
     free(fragments_needed);
@@ -1275,6 +1345,7 @@ static void test_get_fragment_metadata(const ec_backend_id_t be_id, struct ec_ar
         assert(be_version == be->common.ec_backend_version);
     }
     liberasurecode_encode_cleanup(desc, encoded_data, encoded_parity);
+    liberasurecode_instance_destroy(desc);
     free(orig_data);
 }
 
@@ -1357,8 +1428,7 @@ static void test_decode_with_missing_multi_data_parity(
     }
 }
 
-static void test_decode_reconstruct_specific_error_case(
-    const ec_backend_id_t be_id, struct ec_args *args)
+static void test_isa_l_rs_vand_decode_reconstruct_specific_error_case()
 {
     struct ec_args specific_1010_args = {
         .k = 10,
@@ -1387,6 +1457,8 @@ static void test_decode_reconstruct_specific_error_case(
         EC_BACKEND_ISA_L_RS_VAND, &specific_1010_args);
     if (-EBACKENDNOTAVAIL == desc) {
         fprintf (stderr, "Backend library not available!\n");
+        free(orig_data);
+        free(skips);
         return;
     }
     assert(desc > 0);
@@ -1436,6 +1508,7 @@ static void test_decode_reconstruct_specific_error_case(
     assert(rc == 0);
 
     free(out_frag);
+    free(avail_frags);
 
     // cleanup all
     rc = liberasurecode_encode_cleanup(desc, encoded_data, encoded_parity);
@@ -1444,15 +1517,30 @@ static void test_decode_reconstruct_specific_error_case(
     rc = liberasurecode_decode_cleanup(desc, decoded_data);
     assert(rc == 0);
 
-    if (desc) {
-        assert(0 == liberasurecode_instance_destroy(desc));
-    }
-
+    assert(0 == liberasurecode_instance_destroy(desc));
     free(orig_data);
-    free(avail_frags);
     free(skips);
 }
 
+static void test_jerasure_rs_cauchy_init_failure()
+{
+    struct ec_args bad_args = {
+        .k = 10,
+        .m = 10,
+        .w = 4,
+    };
+    // NB: (k + m) > (1 << w) => too many frags!
+
+    int desc = -1;
+    desc = liberasurecode_instance_create(
+        EC_BACKEND_JERASURE_RS_CAUCHY, &bad_args);
+    if (-EBACKENDNOTAVAIL == desc) {
+        fprintf (stderr, "Backend library not available!\n");
+        return;
+    }
+    assert(-EBACKENDINITERR == desc);
+}
+
 static void test_simple_encode_decode(const ec_backend_id_t be_id,
                                      struct ec_args *args)
 {
@@ -1510,6 +1598,8 @@ static void test_verify_stripe_metadata(const ec_backend_id_t be_id,
 
     if (-EBACKENDNOTAVAIL == desc) {
         fprintf (stderr, "Backend library not available!\n");
+        free(orig_data);
+        free(skip);
         return;
     }
     assert(desc > 0);
@@ -1527,8 +1617,10 @@ static void test_verify_stripe_metadata(const ec_backend_id_t be_id,
     assert(0 == rc);
 
     liberasurecode_encode_cleanup(desc, encoded_data, encoded_parity);
+    liberasurecode_instance_destroy(desc);
     free(orig_data);
     free(skip);
+    free(avail_frags);
 }
 
 static void verify_fragment_metadata_mismatch_impl(const ec_backend_id_t be_id, struct ec_args *args,
@@ -1551,6 +1643,8 @@ static void verify_fragment_metadata_mismatch_impl(const ec_backend_id_t be_id,
 
     if (-EBACKENDNOTAVAIL == desc) {
         fprintf (stderr, "Backend library not available!\n");
+        free(orig_data);
+        free(skip);
         return;
     }
     assert(desc > 0);
@@ -1615,6 +1709,8 @@ static void verify_fragment_metadata_mismatch_impl(const ec_backend_id_t be_id,
         }
     }
     liberasurecode_encode_cleanup(desc, encoded_data, encoded_parity);
+    liberasurecode_instance_destroy(desc);
+    free(avail_frags);
     free(orig_data);
     free(skip);
 }
@@ -1791,10 +1887,6 @@ struct testcase testcases[] = {
         test_verify_stripe_metadata_frag_idx_invalid,
         EC_BACKEND_FLAT_XOR_HD, CHKSUM_CRC32,
         .skip = false},
-    {"test_simple_encode_decode_over32",
-        test_simple_encode_decode_over32,
-        EC_BACKEND_JERASURE_RS_VAND, CHKSUM_CRC32,
-        .skip = false},
     // Jerasure RS Vand backend tests
     {"create_and_destroy_backend",
         test_create_and_destroy_backend,
@@ -1860,6 +1952,10 @@ struct testcase testcases[] = {
         test_verify_stripe_metadata_frag_idx_invalid,
         EC_BACKEND_JERASURE_RS_VAND, CHKSUM_CRC32,
         .skip = false},
+    {"test_simple_encode_decode_over32_jerasure_rs_vand",
+        test_simple_encode_decode_over32,
+        EC_BACKENDS_MAX, 0, // NB: this is hardcoded to use jerasure RS vand
+        .skip = false},
     // Jerasure RS Cauchy backend tests
     {"create_and_destroy_backend",
         test_create_and_destroy_backend,
@@ -1925,6 +2021,10 @@ struct testcase testcases[] = {
         test_verify_stripe_metadata_frag_idx_invalid,
         EC_BACKEND_JERASURE_RS_CAUCHY, CHKSUM_CRC32,
         .skip = false},
+    {"test_jerasure_rs_cauchy_init_failure",
+        test_jerasure_rs_cauchy_init_failure,
+        EC_BACKENDS_MAX, 0,
+        .skip = false},
     // ISA-L rs_vand tests
     {"create_and_destroy_backend",
         test_create_and_destroy_backend,
@@ -1986,10 +2086,10 @@ struct testcase testcases[] = {
         test_verify_stripe_metadata_frag_idx_invalid,
         EC_BACKEND_ISA_L_RS_VAND, CHKSUM_CRC32,
         .skip = false},
-    {"test_isa_l_decode_reconstruct_specific_error_case",
-        test_decode_reconstruct_specific_error_case,
+    {"test_isa_l_rs_vand_decode_reconstruct_specific_error_case",
+        test_isa_l_rs_vand_decode_reconstruct_specific_error_case,
         EC_BACKENDS_MAX, 0, // note this test is using ISA-L in hard coded
-	.skip = false},
+        .skip = false},
     // ISA-L rs cauchy tests
     {"create_and_destroy_backend",
         test_create_and_destroy_backend,
@@ -2177,6 +2277,75 @@ struct testcase testcases[] = {
         test_verify_stripe_metadata_frag_idx_invalid,
         EC_BACKEND_LIBERASURECODE_RS_VAND, CHKSUM_CRC32,
         .skip = false},
+    // libphazr backend tests
+    {"create_and_destroy_backend",
+        test_create_and_destroy_backend,
+        EC_BACKEND_LIBPHAZR, CHKSUM_NONE,
+        .skip = false},
+    {"simple_encode_libphazr",
+        test_simple_encode_decode,
+        EC_BACKEND_LIBPHAZR, CHKSUM_NONE,
+        .skip = false},
+    {"decode_with_missing_data_libphazr",
+        test_decode_with_missing_data,
+        EC_BACKEND_LIBPHAZR, CHKSUM_NONE,
+        .skip = false},
+    {"decode_with_missing_parity_libphazr",
+        test_decode_with_missing_parity,
+        EC_BACKEND_LIBPHAZR, CHKSUM_NONE,
+        .skip = false},
+    {"decode_with_missing_multi_data_libphazr",
+        test_decode_with_missing_multi_data,
+        EC_BACKEND_LIBPHAZR, CHKSUM_NONE,
+        .skip = false},
+    {"decode_with_missing_multi_parity_libphazr",
+        test_decode_with_missing_multi_parity,
+        EC_BACKEND_LIBPHAZR, CHKSUM_NONE,
+        .skip = false},
+    {"test_decode_with_missing_multi_data_parity_libphazr",
+        test_decode_with_missing_multi_data_parity,
+        EC_BACKEND_LIBPHAZR, CHKSUM_NONE,
+        .skip = false},
+    {"simple_reconstruct_libphazr",
+        test_simple_reconstruct,
+        EC_BACKEND_LIBPHAZR, CHKSUM_NONE,
+        .skip = false},
+    {"test_fragments_needed_libphazr",
+        test_fragments_needed,
+        EC_BACKEND_LIBPHAZR, CHKSUM_NONE,
+        .skip = false},
+    {"test_get_fragment_metadata_libphazr",
+        test_get_fragment_metadata,
+        EC_BACKEND_LIBPHAZR, CHKSUM_NONE,
+        .skip = false},
+    {"test_get_fragment_metadata_libphazr_crc32",
+        test_get_fragment_metadata,
+        EC_BACKEND_LIBPHAZR, CHKSUM_CRC32,
+        .skip = false},
+    {"test_verify_stripe_metadata",
+        test_verify_stripe_metadata,
+        EC_BACKEND_LIBPHAZR, CHKSUM_CRC32,
+        .skip = false},
+    {"test_verify_stripe_metadata_libec_mismatch",
+        test_verify_stripe_metadata_libec_mismatch,
+        EC_BACKEND_LIBPHAZR, CHKSUM_CRC32,
+        .skip = false},
+    {"test_verify_stripe_metadata_magic_mismatch",
+        test_verify_stripe_metadata_magic_mismatch,
+        EC_BACKEND_LIBPHAZR, CHKSUM_CRC32,
+        .skip = false},
+    {"test_verify_stripe_metadata_be_id_mismatch",
+        test_verify_stripe_metadata_be_id_mismatch,
+        EC_BACKEND_LIBPHAZR, CHKSUM_CRC32,
+        .skip = false},
+    {"test_verify_stripe_metadata_be_ver_mismatch",
+        test_verify_stripe_metadata_be_ver_mismatch,
+        EC_BACKEND_LIBPHAZR, CHKSUM_CRC32,
+        .skip = false},
+    {"test_verify_stripe_metadata_frag_idx_invalid",
+        test_verify_stripe_metadata_frag_idx_invalid,
+        EC_BACKEND_LIBPHAZR, CHKSUM_CRC32,
+        .skip = false},
     { NULL, NULL, 0, 0, false },
 };
 
@@ -2196,7 +2365,10 @@ int main(int argc, char **argv)
                 continue;
             }
             if (testcases[ii].be_id == EC_BACKENDS_MAX) {
-                /* EC_BACKEND_MAX basically designed for invalid args tests
+                if (0 != i) {
+                    continue;
+                }
+                /* EC_BACKENDS_MAX basically designed for invalid args tests
                  * and not takes the args so call the function w/o args here */
                 testcases[ii].function();
                 fprintf(stdout, "ok %d - %s: %s (idx=%d)\n", num_cases,
